# Poisson-Tweedie: Properties and regression models for count data -----
# Author: Wagner Hugo Bonat LEG/IMADA ----------------------------------
# Script 1 - Properties ------------------------------------------------
# Date: 11/07/2016 -----------------------------------------------------
rm(list=ls())

# Loading extra packages
require(tweedie)
require(statmod)
require(lattice)
require(latticeExtra)
# Loading extra functions
source("PTWfunctions.R")

# Figure 1 - Probability mass function ---------------------------------
par(mfrow = c(3,4), mar=c(2.6, 2.8, 1.2, 0.5), mgp = c(1.6, 0.6, 0))
# Case - power = 1.1
plot_ptweedie(mu = 10, phi = 0.8, power = 1.1, n_sample = 100000, 
              grid_y = c(0:100), n_pts = 40, method = "laguerre", 
              main = "DI = 2; p = 1.1")
plot_ptweedie(mu = 10, phi = 3.2, power = 1.1, n_sample = 100000, 
              grid_y = c(0:100), n_pts = 40, method = "laguerre", 
              main = "DI = 5; p = 1.1")
plot_ptweedie(mu = 10, phi = 7.2, power = 1.1, n_sample = 100000, 
              grid_y = c(0:100), n_pts = 40, method = "laguerre", 
              main = "DI = 10; p = 1.1")
plot_ptweedie(mu = 10, phi = 15, power = 1.1, n_sample = 100000, 
              grid_y = c(0:100), n_pts = 40, method = "laguerre", 
              main = "DI = 20; p = 1.1")

# Case - power = 2
plot_ptweedie(mu = 10, phi = 0.1, power = 2, n_sample = 100000, 
              grid_y = c(0:100), n_pts = 40, method = "laguerre", 
              main = "DI = 2; p = 2" )
plot_ptweedie(mu = 10, phi = 0.4, power = 2, n_sample = 100000, 
              grid_y = c(0:500), n_pts = 40, method = "laguerre", 
              main = "DI = 5; p = 2")
plot_ptweedie(mu = 10, phi = 0.9, power = 2, n_sample = 100000, 
              grid_y = c(0:500), n_pts = 40, method = "laguerre", 
              main = "DI = 10; p = 2")
plot_ptweedie(mu = 10, phi = 1.9, power = 2, n_sample = 100000, 
              grid_y = c(0:800), n_pts = 40, method = "laguerre",
              main = "DI = 20; p = 2")

# Case - power = 3
plot_ptweedie(mu = 10, phi = 0.01, power = 3, n_sample = 100000, 
              grid_y = c(0:100), n_pts = 40, method = "laguerre", 
              main = "DI = 2; p = 3")
plot_ptweedie(mu = 10, phi = 0.04, power = 3, n_sample = 100000, 
              grid_y = c(0:500), n_pts = 40, method = "laguerre", 
              main = "DI = 5; p = 3")
plot_ptweedie(mu = 10, phi = 0.09, power = 3, n_sample = 100000, 
              grid_y = c(0:500), n_pts = 40, method = "laguerre", 
              main = "DI = 10; p = 3")
plot_ptweedie(mu = 10, phi = 0.19, power = 3, n_sample = 100000, 
              grid_y = c(0:800), n_pts = 40, method = "laguerre", 
              main = "DI = 20; p = 3")

# Figure 2 - Dispersion index + zero inflation index + heavy tail index-
# Regression structure
x1 <- seq(-1,1,l = 100)
mu <- exp(log(10) + 0.8*x1)
x.values <- seq(40, 100, by = 5)
# Case 1.1
DI1_11 <- disp_index(mu = mu, phi = 0.8, p = 1.1)
ZI1_11 <- zero_inflation(mu = mu, phi = 0.8, p = 1.1)
HT1_11 <- long_tail(x = x.values, mu = 10, phi = 0.8, power = 1.1)

DI2_11 <- disp_index(mu = mu, phi = 3.2, p = 1.1)
ZI2_11 <- zero_inflation(mu = mu, phi = 3.2, p = 1.1)
HT2_11 <- long_tail(x = x.values, mu = 10, phi = 3.2, power = 1.1)

DI3_11 <- disp_index(mu = mu, phi = 7.2, p = 1.1)
ZI3_11 <- zero_inflation(mu = mu, phi = 7.2, p = 1.1)
HT3_11 <- long_tail(x = x.values, mu = 10, phi = 7.2, power = 1.1)

DI4_11 <- disp_index(mu = mu, phi = 15, p = 1.1)
ZI4_11 <- zero_inflation(mu = mu, phi = 15, p = 1.1)
HT4_11 <- long_tail(x = x.values, mu = 10, phi = 15, power = 1.1)

# Case 2
DI1_2 <- disp_index(mu = mu, phi = 0.1, p = 2)
ZI1_2 <- zero_inflation(mu = mu, phi = 0.1, p = 2)
HT1_2 <- long_tail(x = x.values, mu = 10, phi = 0.1, power = 2)

DI2_2 <- disp_index(mu = mu, phi = 0.4, p = 2)
ZI2_2 <- zero_inflation(mu = mu, phi = 0.4, p = 2)
HT2_2 <- long_tail(x = x.values, mu = 10, phi = 0.4, power = 2)

DI3_2 <- disp_index(mu = mu, phi = 0.9, p = 2)
ZI3_2 <- zero_inflation(mu = mu, phi = 0.9, p = 2)
HT3_2 <- long_tail(x = x.values, mu = 10, phi = 0.9, power = 2)

DI4_2 <- disp_index(mu = mu, phi = 1.9, p = 2)
ZI4_2 <- zero_inflation(mu = mu, phi = 1.9, p = 2)
HT4_2 <- long_tail(x = x.values, mu = 10, phi = 1.9, power = 2)

# Case 3
DI1_3 <- disp_index(mu = mu, phi = 0.01, p = 3)
ZI1_3 <- zero_inflation(mu = mu, phi = 0.01, p = 3)
HT1_3 <- long_tail(x = x.values, mu = 10, phi = 0.01, power = 3)

DI2_3 <- disp_index(mu = mu, phi = 0.04, p = 3)
ZI2_3 <- zero_inflation(mu = mu, phi = 0.04, p = 3)
HT2_3 <- long_tail(x = x.values, mu = 10, phi = 0.04, power = 3)

DI3_3 <- disp_index(mu = mu, phi = 0.09, p = 3)
ZI3_3 <- zero_inflation(mu = mu, phi = 0.09, p = 3)
HT3_3 <- long_tail(x = x.values, mu = 10, phi = 0.09, power = 3)

DI4_3 <- disp_index(mu = mu, phi = 0.19, p = 3)
ZI4_3 <- zero_inflation(mu = mu, phi = 0.19, p = 3)
HT4_3 <- long_tail(x = x.values, mu = 10, phi = 0.19, power = 3)

# Preparing the data sets to plot using lattice package ----------------

# Dispersion index
DI1 <- data.frame("ID" = c(DI1_11, DI1_2, DI1_3),
                  "Mu" = c(mu,mu,mu),
                  "POWER" = rep(c(1.1,2,3), each = length(mu)),
                  "Disp" = 1, "TYPE" = "DI")
DI2 <- data.frame("ID" = c(DI2_11, DI2_2, DI2_3),
                  "Mu" = c(mu,mu,mu),
                  "POWER" = rep(c(1.1,2,3), each = length(mu)),
                  "Disp" = 2, "TYPE" = "DI")
DI3 <- data.frame("ID" = c(DI3_11, DI3_2, DI3_3),
                  "Mu" = c(mu,mu,mu),
                  "POWER" = rep(c(1.1,2,3), each = length(mu)),
                  "Disp" = 3, "TYPE" = "DI")
DI4 <- data.frame("ID" = c(DI4_11, DI4_2, DI4_3),
                  "Mu" = c(mu,mu,mu),
                  "POWER" = rep(c(1.1,2,3), each = length(mu)),
                  "Disp" = 4, "TYPE" = "DI")
DI <- rbind(DI1,DI2,DI3,DI4)

# Zero-inflation index
ZI1 <- data.frame("ID" = c(ZI1_11, ZI1_2, ZI1_3),
                  "Mu" = c(mu,mu,mu),
                  "POWER" = rep(c(1.1,2,3), each = length(mu)),
                  "Disp" = 1, "TYPE" = "ZI")
ZI2 <- data.frame("ID" = c(ZI2_11, ZI2_2, ZI2_3),
                  "Mu" = c(mu,mu,mu),
                  "POWER" = rep(c(1.1,2,3), each = length(mu)),
                  "Disp" = 2, "TYPE" = "ZI")
ZI3 <- data.frame("ID" = c(ZI3_11, ZI3_2, ZI3_3),
                  "Mu" = c(mu,mu,mu),
                  "POWER" = rep(c(1.1,2,3), each = length(mu)),
                  "Disp" = 3, "TYPE" = "ZI")
ZI4 <- data.frame("ID" = c(ZI4_11, ZI4_2, ZI4_3),
                  "Mu" = c(mu,mu,mu),
                  "POWER" = rep(c(1.1,2,3), each = length(mu)),
                  "Disp" = 4, "TYPE" = "ZI")
ZI <- rbind(ZI1,ZI2,ZI3,ZI4)

# Heavy-tail index
HT1 <- data.frame("ID" = c(HT1_11, HT1_2, HT1_3),
                  "Mu" = c(x.values,x.values,x.values),
                  "POWER" = rep(c(1.1,2,3), each = length(x.values)),
                  "Disp" = 1, "TYPE" = "HT")
HT2 <- data.frame("ID" = c(HT2_11, HT2_2, HT2_3),
                  "Mu" = c(x.values,x.values,x.values),
                  "POWER" = rep(c(1.1,2,3), each = length(x.values)),
                  "Disp" = 2, "TYPE" = "HT")
HT3 <- data.frame("ID" = c(HT3_11, HT3_2, HT3_3),
                  "Mu" = c(x.values,x.values,x.values),
                  "POWER" = rep(c(1.1,2,3), each = length(x.values)),
                  "Disp" = 3, "TYPE" = "HT")
HT4 <- data.frame("ID" = c(HT4_11, HT4_2, HT4_3),
                  "Mu" = c(x.values,x.values,x.values),
                  "POWER" = rep(c(1.1,2,3), each = length(x.values)),
                  "Disp" = 4, "TYPE" = "HT")
HT <- rbind(HT1,HT2,HT3,HT4)

# Dispersion index -----------------------------------------------------
DI$POWER <- as.factor(DI$POWER)
DI$Disp <- as.factor(DI$Disp)
levels(DI$Disp) <- c("Scenario 1","Scenario 2","Scenario 3","Scenario 4")

key <- list(text=list(levels(DI$POWER)),
            title="Power parameter", cex.title=1.1,
            lines=list(pch=c(1,2,15)), 
            col = c("black","gray10","gray40"),
            lty = 1:3,
            divide=c(1), columns=3, type="l")

DI.plot <- xyplot(ID ~ Mu | Disp, ylab = "DI",
                  xlab = expression(mu),
                  data = DI, group = POWER, type="l",
                  par.settings = ps, scales = "free",
                  horizontal = TRUE, 
                  pch = c(1,2,15), 
                  lty = 1:3,
                  layout = c(4,1),
                  col = c("black","gray10","gray40"),
                  cex = 0.5, key = key)

# Zero inflation index -------------------------------------------------
ZI$POWER <- as.factor(ZI$POWER)
ZI$Disp <- as.factor(ZI$Disp)
levels(ZI$Disp) <- c("Scenario 1","Scenario 2","Scenario 3","Scenario 4")

ZI.plot <- xyplot(ID ~ Mu | Disp, ylab = "ZI",
                  xlab = expression(mu),
                  data = ZI, group = POWER, type="l",
                  par.settings = ps, scales = "free",
                  horizontal = TRUE, 
                  pch = c(1,2,15), 
                  lty = 1:3,
                  layout = c(4,1),
                  col = c("black","gray10","gray40"),
                  cex = 0.5)

# Heavy tail index -----------------------------------------------------
HT$POWER <- as.factor(HT$POWER)
HT$Disp <- as.factor(HT$Disp)
levels(HT$Disp) <- c("Scenario 1","Scenario 2","Scenario 3","Scenario 4")

HT.plot <- xyplot(ID ~ Mu | Disp, ylab = "HT",
                  xlab = "y",
                  data = HT, group = POWER, type="l",
                  par.settings = ps, scales = "free",
                  horizontal = TRUE, 
                  pch = c(1,2,15), 
                  lty = 1:3,
                  layout = c(4,1),
                  col = c("black","gray10","gray40"),
                  cex = 0.5)
# Figure 2
print(DI.plot, split=c(1,1,1,3), more=TRUE)
print(ZI.plot, split=c(1,2,1,3), more = TRUE)
print(HT.plot, split=c(1,3,1,3))


# Figure 3
par(mfrow = c(1,3), mar=c(2.6, 2.8, 1.2, 0.5), mgp = c(1.6, 0.6, 0))
plot(disp_index(mu = mu, phi = -0.7, power = 1.1) ~ mu, type = "l", 
     ylab = "DI", xlab = expression(mu), ylim = c(0, 1), main = "p = 1.1")
text(x = 20 ,y = 0.08, labels = "-0.7")
lines(mu, disp_index(mu = mu, phi = -0.5, power = 1.1), lty = 2)
text(x = 20 ,y = 0.35, labels = "-0.5")
lines(mu, disp_index(mu = mu, phi = -0.3, power = 1.1), lty = 3)
text(x = 20 ,y = 0.62, labels = "-0.3")
lines(mu, disp_index(mu = mu, phi = -0.1, power = 1.1), lty = 4)
text(x = 20 ,y = 0.89, labels = "-0.1")

plot(disp_index(mu = mu, phi = -0.04, power = 2) ~ mu, type = "l", 
     ylab = "DI", xlab = expression(mu), ylim = c(0, 1), main = "p = 2")
text(x = 19,y = 0.12, labels = "-0.04")
lines(mu, disp_index(mu = mu, phi = -0.02, power = 2), lty = 2)
text(x = 19,y = 0.55, labels = "-0.02")
lines(mu, disp_index(mu = mu, phi = -0.01, power = 2), lty = 3)
text(x = 19,y = 0.77, labels = "-0.01")
lines(mu, disp_index(mu = mu, phi = -0.005, power = 2), lty = 4)
text(x = 19,y = 0.86, labels = "-0.005")

plot(disp_index(mu = mu, phi = -0.002, power = 3) ~ mu, type = "l", 
     ylab = "DI", xlab = expression(mu), ylim = c(0, 1), main = "p = 3")
text(x = 18.5,y = 0.1, labels = "-0.002")
lines(mu, disp_index(mu = mu, phi = -0.001, power = 3), lty = 2)
text(x = 19.5,y = 0.5, labels = "-0.001")
lines(mu, disp_index(mu = mu, phi = -0.0005, power = 3), lty = 3)
text(x = 19.5,y = 0.75, labels = "-0.0005")
lines(mu, disp_index(mu = mu, phi = -0.0001, power = 3), lty = 4)
text(x = 19,y = 0.93, labels = "-0.0001")
# END ------------------------------------------------------------------
save.image("Script1.RData")
